// -*- mode:c++; c-style:k&r; c-basic-offset:4; indent-tabs-mode: nil; -*-
// vi:set et cin sw=4 cino=>se0n0f0{0}0^0\:0=sl1g0hspst0+sc3C0/0(0u0U0w0m0:

#ifndef ENCODER_HPP
#define ENCODER_HPP

#include "Common.hpp"
#include "Vocabulary.hpp"

// state: move (i.e., previous cell)
// score: current score


// x, rot >- dpxcoord
//
// x-coord

class Encoder {
private:
    class EncoderState {
    public:
        bool is_orig() { return orig; };
        bool is_known() { return score >= 0; };
        bool is_invalid() { return score == -1; };
    public:
        int32_t score;
        int move_no;
        Pos prev;
        bool orig;
    };
    const EncoderState unknown = { -10000, 0, std::make_tuple(0,0,0), false };
    const EncoderState invalid = { -1, 0, std::make_tuple(0,0,0), false };
    const EncoderState original = { -10000, -1, std::make_tuple(0,0,0), true };
    const int num_rotations = 6;

public:
    Encoder(int width, int num_lines, const Vocabulary &vocab,
            const std::vector<const Frame> &frames);
    // returns <score,encoding>
    std::tuple<int,std::string> run();
private:
    const int m_width;
    const int m_height;
    const int m_num_pieces;
    const Vocabulary &m_vocab;
    // indices: frame#, final#, prim_move#
    std::vector<const Pos> m_frame_spawns;
    std::vector<std::vector<std::vector<Pos> > > m_frame_final_locs;
    std::vector<int> m_frame_rot_lims;
    std::vector<std::vector<int> > m_frame_x_mins;
    std::vector<std::vector<int> > m_frame_y_mins;
    std::vector<EncoderState> m_memo;
private:
    Delta adj_delta(Delta delta, int end_step);
    EncoderState &memo_at(int step, int x, int rot);
    // returns <valid?, final_x, final_rot, final_y>
    std::vector<Pos> validate_move(int cur_x, int cur_rot, int cur_step, const Word &m);
    std::vector<Pos> validate_move_rec(int cur_x, int cur_rot, int cur_step, 
                                       std::vector<const Delta>::const_iterator d_cur,
                                       std::vector<const Delta>::const_iterator d_end);
    void compute_memo_table();
    std::tuple<int,std::string> trace_solution();
};

inline Encoder::EncoderState &Encoder::memo_at(int step, int x, int rot) {
    return m_memo[step * 6 * m_width + 6 * x + rot];
}

#endif // ENCODER_HPP

