#ifndef GAME_HPP
#define GAME_HPP

#include "Common.hpp"
#include "Grid.hpp"
#include "Problem.hpp"
#include "Source.hpp"
#include "Command.hpp"
#include "UnitState.hpp"

class Ansi;
class Move;

class UnitInfo
{
public:
    UnitInfo() : m_initState(), m_rots(0), m_xMins(), m_yMins() {}

public:
    UnitState m_initState;
    uint32_t m_rots;
    std::vector<int32_t> m_xMins;
    std::vector<int32_t> m_yMins;
};

class Game
{
public:
    Game();

    void init(const Problem& problem,
              const std::vector<std::string>& phrases,
              uint32_t seed);
    UnitState transformUnit(Command command,
                            uint32_t unitIndex,
                            UnitState unitState) const;
    UnitState untransformUnit(Command command,
                              uint32_t unitIndex,
                              UnitState unitState) const;
    bool isInRange(uint32_t unitIndex, UnitState unitState) const;
    bool isValid(uint32_t unitIndex, UnitState unitState) const;
    void spawn();
    void step(char ch);
    void step(Command command);
    void move(UnitState unitState);
    void finishMove();
    void resetVisitedStates();
    void updateVisitedStates();
    void print(Ansi& ansi) const;

public:
    std::vector<std::string> m_phrases;
    std::vector<char> m_chars;
    std::vector<uint32_t> m_phraseCounts;
    Grid<uint32_t> m_grid;
    std::vector<Unit> m_units;
    std::vector<UnitInfo> m_unitInfo;
    std::vector<std::vector<uint8_t>> m_visitedStates;
    uint32_t m_curUnitIndex;
    UnitState m_curUnitState;
    bool m_done;
    bool m_checkVisitedStates;
    bool m_repeatedStates;
    uint64_t m_sourceLength;
    Source m_source;
    uint64_t m_spawnCount;
    uint64_t m_score;
    uint64_t m_moveScore;
    uint64_t m_powerScore;
    uint64_t m_linesOld;
};

#endif
