#ifndef KEY_HPP
#define KEY_HPP

#include "Common.hpp"

enum class Key : uint32_t
{
    Invalid = 0x00,

    Shift = 0x100,
    Ctrl = 0x200,
    Alt = 0x400,

    Backspace = 0x08,
    Tab = 0x09,
    Center = 0x0c,
    Enter = 0x0d,
    Esc = 0x1b,
    Space = 0x20,
    PgUp = 0x21,
    PgDn = 0x22,
    End = 0x23,
    Home = 0x24,
    Left = 0x25,
    Up = 0x26,
    Right = 0x27,
    Down = 0x28,
    Insert = 0x2d,
    Delete = 0x2e,

    Key0 = 0x30,
    Key1 = 0x31,
    Key2 = 0x32,
    Key3 = 0x33,
    Key4 = 0x34,
    Key5 = 0x35,
    Key6 = 0x36,
    Key7 = 0x37,
    Key8 = 0x38,
    Key9 = 0x39,

    A = 0x41,
    B = 0x42,
    C = 0x43,
    D = 0x44,
    E = 0x45,
    F = 0x46,
    G = 0x47,
    H = 0x48,
    I = 0x49,
    J = 0x4a,
    K = 0x4b,
    L = 0x4c,
    M = 0x4d,
    N = 0x4e,
    O = 0x4f,
    P = 0x50,
    Q = 0x51,
    R = 0x52,
    S = 0x53,
    T = 0x54,
    U = 0x55,
    V = 0x56,
    W = 0x57,
    X = 0x58,
    Y = 0x59,
    Z = 0x5a,

    F1 = 0x70,
    F2 = 0x71,
    F3 = 0x72,
    F4 = 0x73,
    F5 = 0x74,
    F6 = 0x75,
    F7 = 0x76,
    F8 = 0x77,
    F9 = 0x78,
    F10 = 0x79,
    F11 = 0x7a,
    F12 = 0x7b,

    Semicolon = 0xba,
    Equals = 0xbb,
    Comma = 0xbc,
    Minus = 0xbd,
    Period = 0xbe,
    Slash = 0xbf,
    Backtick = 0xc0,
    LeftBracket = 0xdb,
    Backslash = 0xdc,
    RightBracket = 0xdd,
    Apostrophe = 0xde,

    ShiftBackspace = Shift | Backspace,
    ShiftTab = Shift | Tab,
    ShiftCenter = Shift | Center,
    ShiftEnter = Shift | Enter,
    ShiftEsc = Shift | Esc,
    ShiftSpace = Shift | Space,
    ShiftPgUp = Shift | PgUp,
    ShiftPgDn = Shift | PgDn,
    ShiftEnd = Shift | End,
    ShiftHome = Shift | Home,
    ShiftLeft = Shift | Left,
    ShiftUp = Shift | Up,
    ShiftRight = Shift | Right,
    ShiftDown = Shift | Down,
    ShiftInsert = Shift | Insert,
    ShiftDelete = Shift | Delete,

    Shift0 = Shift | Key0,
    Shift1 = Shift | Key1,
    Shift2 = Shift | Key2,
    Shift3 = Shift | Key3,
    Shift4 = Shift | Key4,
    Shift5 = Shift | Key5,
    Shift6 = Shift | Key6,
    Shift7 = Shift | Key7,
    Shift8 = Shift | Key8,
    Shift9 = Shift | Key9,

    Exclamation = Shift1,
    At = Shift2,
    Pound = Shift3,
    Dollar = Shift4,
    Percent = Shift5,
    Caret = Shift6,
    Ampersand = Shift7,
    Asterisk = Shift8,
    LeftParen = Shift9,
    RightParen = Shift0,

    ShiftA = Shift | A,
    ShiftB = Shift | B,
    ShiftC = Shift | C,
    ShiftD = Shift | D,
    ShiftE = Shift | E,
    ShiftF = Shift | F,
    ShiftG = Shift | G,
    ShiftH = Shift | H,
    ShiftI = Shift | I,
    ShiftJ = Shift | J,
    ShiftK = Shift | K,
    ShiftL = Shift | L,
    ShiftM = Shift | M,
    ShiftN = Shift | N,
    ShiftO = Shift | O,
    ShiftP = Shift | P,
    ShiftQ = Shift | Q,
    ShiftR = Shift | R,
    ShiftS = Shift | S,
    ShiftT = Shift | T,
    ShiftU = Shift | U,
    ShiftV = Shift | V,
    ShiftW = Shift | W,
    ShiftX = Shift | X,
    ShiftY = Shift | Y,
    ShiftZ = Shift | Z,

    ShiftF1 = Shift | F1,
    ShiftF2 = Shift | F2,
    ShiftF3 = Shift | F3,
    ShiftF4 = Shift | F4,
    ShiftF5 = Shift | F5,
    ShiftF6 = Shift | F6,
    ShiftF7 = Shift | F7,
    ShiftF8 = Shift | F8,
    ShiftF9 = Shift | F9,
    ShiftF10 = Shift | F10,
    ShiftF11 = Shift | F11,
    ShiftF12 = Shift | F12,

    ShiftSemicolon = Shift | Semicolon,
    ShiftEquals = Shift | Equals,
    ShiftComma = Shift | Comma,
    ShiftMinus = Shift | Minus,
    ShiftPeriod = Shift | Period,
    ShiftSlash = Shift | Slash,
    ShiftBacktick = Shift | Backtick,
    ShiftLeftBracket = Shift | LeftBracket,
    ShiftBackslash = Shift | Backslash,
    ShiftRightBracket = Shift | RightBracket,
    ShiftApostrophe = Shift | Apostrophe,

    Colon = ShiftSemicolon,
    Plus = ShiftEquals,
    Less = ShiftComma,
    Underscore = ShiftMinus,
    Greater = ShiftPeriod,
    Question = ShiftSlash,
    Tilde = ShiftBacktick,
    LeftBrace = ShiftLeftBracket,
    VerticalBar = ShiftBackslash,
    RightBrace = ShiftRightBracket,
    Quote = ShiftApostrophe,

    CtrlBackspace = Ctrl | Backspace,
    CtrlTab = Ctrl | Tab,
    CtrlCenter = Ctrl | Center,
    CtrlEnter = Ctrl | Enter,
    CtrlEsc = Ctrl | Esc,
    CtrlSpace = Ctrl | Space,
    CtrlPgUp = Ctrl | PgUp,
    CtrlPgDn = Ctrl | PgDn,
    CtrlEnd = Ctrl | End,
    CtrlHome = Ctrl | Home,
    CtrlLeft = Ctrl | Left,
    CtrlUp = Ctrl | Up,
    CtrlRight = Ctrl | Right,
    CtrlDown = Ctrl | Down,
    CtrlInsert = Ctrl | Insert,
    CtrlDelete = Ctrl | Delete,

    Ctrl0 = Ctrl | Key0,
    Ctrl1 = Ctrl | Key1,
    Ctrl2 = Ctrl | Key2,
    Ctrl3 = Ctrl | Key3,
    Ctrl4 = Ctrl | Key4,
    Ctrl5 = Ctrl | Key5,
    Ctrl6 = Ctrl | Key6,
    Ctrl7 = Ctrl | Key7,
    Ctrl8 = Ctrl | Key8,
    Ctrl9 = Ctrl | Key9,

    CtrlA = Ctrl | A,
    CtrlB = Ctrl | B,
    CtrlC = Ctrl | C,
    CtrlD = Ctrl | D,
    CtrlE = Ctrl | E,
    CtrlF = Ctrl | F,
    CtrlG = Ctrl | G,
    CtrlH = Ctrl | H,
    CtrlI = Ctrl | I,
    CtrlJ = Ctrl | J,
    CtrlK = Ctrl | K,
    CtrlL = Ctrl | L,
    CtrlM = Ctrl | M,
    CtrlN = Ctrl | N,
    CtrlO = Ctrl | O,
    CtrlP = Ctrl | P,
    CtrlQ = Ctrl | Q,
    CtrlR = Ctrl | R,
    CtrlS = Ctrl | S,
    CtrlT = Ctrl | T,
    CtrlU = Ctrl | U,
    CtrlV = Ctrl | V,
    CtrlW = Ctrl | W,
    CtrlX = Ctrl | X,
    CtrlY = Ctrl | Y,
    CtrlZ = Ctrl | Z,

    CtrlF1 = Ctrl | F1,
    CtrlF2 = Ctrl | F2,
    CtrlF3 = Ctrl | F3,
    CtrlF4 = Ctrl | F4,
    CtrlF5 = Ctrl | F5,
    CtrlF6 = Ctrl | F6,
    CtrlF7 = Ctrl | F7,
    CtrlF8 = Ctrl | F8,
    CtrlF9 = Ctrl | F9,
    CtrlF10 = Ctrl | F10,
    CtrlF11 = Ctrl | F11,
    CtrlF12 = Ctrl | F12,

    CtrlSemicolon = Ctrl | Semicolon,
    CtrlEquals = Ctrl | Equals,
    CtrlComma = Ctrl | Comma,
    CtrlMinus = Ctrl | Minus,
    CtrlPeriod = Ctrl | Period,
    CtrlSlash = Ctrl | Slash,
    CtrlBacktick = Ctrl | Backtick,
    CtrlLeftBracket = Ctrl | LeftBracket,
    CtrlBackslash = Ctrl | Backslash,
    CtrlRightBracket = Ctrl | RightBracket,
    CtrlApostrophe = Ctrl | Apostrophe,

    CtrlShiftBackspace = Ctrl | Shift | Backspace,
    CtrlShiftTab = Ctrl | Shift | Tab,
    CtrlShiftCenter = Ctrl | Shift | Center,
    CtrlShiftEnter = Ctrl | Shift | Enter,
    CtrlShiftEsc = Ctrl | Shift | Esc,
    CtrlShiftSpace = Ctrl | Shift | Space,
    CtrlShiftPgUp = Ctrl | Shift | PgUp,
    CtrlShiftPgDn = Ctrl | Shift | PgDn,
    CtrlShiftEnd = Ctrl | Shift | End,
    CtrlShiftHome = Ctrl | Shift | Home,
    CtrlShiftLeft = Ctrl | Shift | Left,
    CtrlShiftUp = Ctrl | Shift | Up,
    CtrlShiftRight = Ctrl | Shift | Right,
    CtrlShiftDown = Ctrl | Shift | Down,
    CtrlShiftInsert = Ctrl | Shift | Insert,
    CtrlShiftDelete = Ctrl | Shift | Delete,

    CtrlShift0 = Ctrl | Shift | Key0,
    CtrlShift1 = Ctrl | Shift | Key1,
    CtrlShift2 = Ctrl | Shift | Key2,
    CtrlShift3 = Ctrl | Shift | Key3,
    CtrlShift4 = Ctrl | Shift | Key4,
    CtrlShift5 = Ctrl | Shift | Key5,
    CtrlShift6 = Ctrl | Shift | Key6,
    CtrlShift7 = Ctrl | Shift | Key7,
    CtrlShift8 = Ctrl | Shift | Key8,
    CtrlShift9 = Ctrl | Shift | Key9,

    CtrlExclamation = CtrlShift1,
    CtrlAt = CtrlShift2,
    CtrlPound = CtrlShift3,
    CtrlDollar = CtrlShift4,
    CtrlPercent = CtrlShift5,
    CtrlCaret = CtrlShift6,
    CtrlAmpersand = CtrlShift7,
    CtrlAsterisk = CtrlShift8,
    CtrlLeftParen = CtrlShift9,
    CtrlRightParen = CtrlShift0,

    CtrlShiftA = Ctrl | Shift | A,
    CtrlShiftB = Ctrl | Shift | B,
    CtrlShiftC = Ctrl | Shift | C,
    CtrlShiftD = Ctrl | Shift | D,
    CtrlShiftE = Ctrl | Shift | E,
    CtrlShiftF = Ctrl | Shift | F,
    CtrlShiftG = Ctrl | Shift | G,
    CtrlShiftH = Ctrl | Shift | H,
    CtrlShiftI = Ctrl | Shift | I,
    CtrlShiftJ = Ctrl | Shift | J,
    CtrlShiftK = Ctrl | Shift | K,
    CtrlShiftL = Ctrl | Shift | L,
    CtrlShiftM = Ctrl | Shift | M,
    CtrlShiftN = Ctrl | Shift | N,
    CtrlShiftO = Ctrl | Shift | O,
    CtrlShiftP = Ctrl | Shift | P,
    CtrlShiftQ = Ctrl | Shift | Q,
    CtrlShiftR = Ctrl | Shift | R,
    CtrlShiftS = Ctrl | Shift | S,
    CtrlShiftT = Ctrl | Shift | T,
    CtrlShiftU = Ctrl | Shift | U,
    CtrlShiftV = Ctrl | Shift | V,
    CtrlShiftW = Ctrl | Shift | W,
    CtrlShiftX = Ctrl | Shift | X,
    CtrlShiftY = Ctrl | Shift | Y,
    CtrlShiftZ = Ctrl | Shift | Z,

    CtrlShiftF1 = Ctrl | Shift | F1,
    CtrlShiftF2 = Ctrl | Shift | F2,
    CtrlShiftF3 = Ctrl | Shift | F3,
    CtrlShiftF4 = Ctrl | Shift | F4,
    CtrlShiftF5 = Ctrl | Shift | F5,
    CtrlShiftF6 = Ctrl | Shift | F6,
    CtrlShiftF7 = Ctrl | Shift | F7,
    CtrlShiftF8 = Ctrl | Shift | F8,
    CtrlShiftF9 = Ctrl | Shift | F9,
    CtrlShiftF10 = Ctrl | Shift | F10,
    CtrlShiftF11 = Ctrl | Shift | F11,
    CtrlShiftF12 = Ctrl | Shift | F12,

    CtrlShiftSemicolon = Ctrl | Shift | Semicolon,
    CtrlShiftEquals = Ctrl | Shift | Equals,
    CtrlShiftComma = Ctrl | Shift | Comma,
    CtrlShiftMinus = Ctrl | Shift | Minus,
    CtrlShiftPeriod = Ctrl | Shift | Period,
    CtrlShiftSlash = Ctrl | Shift | Slash,
    CtrlShiftBacktick = Ctrl | Shift | Backtick,
    CtrlShiftLeftBracket = Ctrl | Shift | LeftBracket,
    CtrlShiftBackslash = Ctrl | Shift | Backslash,
    CtrlShiftRightBracket = Ctrl | Shift | RightBracket,
    CtrlShiftApostrophe = Ctrl | Shift | Apostrophe,

    CtrlColon = CtrlShiftSemicolon,
    CtrlPlus = CtrlShiftEquals,
    CtrlLess = CtrlShiftComma,
    CtrlUnderscore = CtrlShiftMinus,
    CtrlGreater = CtrlShiftPeriod,
    CtrlQuestion = CtrlShiftSlash,
    CtrlTilde = CtrlShiftBacktick,
    CtrlLeftBrace = CtrlShiftLeftBracket,
    CtrlVerticalBar = CtrlShiftBackslash,
    CtrlRightBrace = CtrlShiftRightBracket,
    CtrlQuote = CtrlShiftApostrophe,

    AltBackspace = Alt | Backspace,
    AltTab = Alt | Tab,
    AltCenter = Alt | Center,
    AltEnter = Alt | Enter,
    AltEsc = Alt | Esc,
    AltSpace = Alt | Space,
    AltPgUp = Alt | PgUp,
    AltPgDn = Alt | PgDn,
    AltEnd = Alt | End,
    AltHome = Alt | Home,
    AltLeft = Alt | Left,
    AltUp = Alt | Up,
    AltRight = Alt | Right,
    AltDown = Alt | Down,
    AltInsert = Alt | Insert,
    AltDelete = Alt | Delete,

    Alt0 = Alt | Key0,
    Alt1 = Alt | Key1,
    Alt2 = Alt | Key2,
    Alt3 = Alt | Key3,
    Alt4 = Alt | Key4,
    Alt5 = Alt | Key5,
    Alt6 = Alt | Key6,
    Alt7 = Alt | Key7,
    Alt8 = Alt | Key8,
    Alt9 = Alt | Key9,

    AltA = Alt | A,
    AltB = Alt | B,
    AltC = Alt | C,
    AltD = Alt | D,
    AltE = Alt | E,
    AltF = Alt | F,
    AltG = Alt | G,
    AltH = Alt | H,
    AltI = Alt | I,
    AltJ = Alt | J,
    AltK = Alt | K,
    AltL = Alt | L,
    AltM = Alt | M,
    AltN = Alt | N,
    AltO = Alt | O,
    AltP = Alt | P,
    AltQ = Alt | Q,
    AltR = Alt | R,
    AltS = Alt | S,
    AltT = Alt | T,
    AltU = Alt | U,
    AltV = Alt | V,
    AltW = Alt | W,
    AltX = Alt | X,
    AltY = Alt | Y,
    AltZ = Alt | Z,

    AltF1 = Alt | F1,
    AltF2 = Alt | F2,
    AltF3 = Alt | F3,
    AltF4 = Alt | F4,
    AltF5 = Alt | F5,
    AltF6 = Alt | F6,
    AltF7 = Alt | F7,
    AltF8 = Alt | F8,
    AltF9 = Alt | F9,
    AltF10 = Alt | F10,
    AltF11 = Alt | F11,
    AltF12 = Alt | F12,

    AltSemicolon = Alt | Semicolon,
    AltEquals = Alt | Equals,
    AltComma = Alt | Comma,
    AltMinus = Alt | Minus,
    AltPeriod = Alt | Period,
    AltSlash = Alt | Slash,
    AltBacktick = Alt | Backtick,
    AltLeftBracket = Alt | LeftBracket,
    AltBackslash = Alt | Backslash,
    AltRightBracket = Alt | RightBracket,
    AltApostrophe = Alt | Apostrophe,

    AltShiftBackspace = Alt | Shift | Backspace,
    AltShiftTab = Alt | Shift | Tab,
    AltShiftCenter = Alt | Shift | Center,
    AltShiftEnter = Alt | Shift | Enter,
    AltShiftEsc = Alt | Shift | Esc,
    AltShiftSpace = Alt | Shift | Space,
    AltShiftPgUp = Alt | Shift | PgUp,
    AltShiftPgDn = Alt | Shift | PgDn,
    AltShiftEnd = Alt | Shift | End,
    AltShiftHome = Alt | Shift | Home,
    AltShiftLeft = Alt | Shift | Left,
    AltShiftUp = Alt | Shift | Up,
    AltShiftRight = Alt | Shift | Right,
    AltShiftDown = Alt | Shift | Down,
    AltShiftInsert = Alt | Shift | Insert,
    AltShiftDelete = Alt | Shift | Delete,

    AltShift0 = Alt | Shift | Key0,
    AltShift1 = Alt | Shift | Key1,
    AltShift2 = Alt | Shift | Key2,
    AltShift3 = Alt | Shift | Key3,
    AltShift4 = Alt | Shift | Key4,
    AltShift5 = Alt | Shift | Key5,
    AltShift6 = Alt | Shift | Key6,
    AltShift7 = Alt | Shift | Key7,
    AltShift8 = Alt | Shift | Key8,
    AltShift9 = Alt | Shift | Key9,

    AltExclamation = AltShift1,
    AltAt = AltShift2,
    AltPound = AltShift3,
    AltDollar = AltShift4,
    AltPercent = AltShift5,
    AltCaret = AltShift6,
    AltAmpersand = AltShift7,
    AltAsterisk = AltShift8,
    AltLeftParen = AltShift9,
    AltRightParen = AltShift0,

    AltShiftA = Alt | Shift | A,
    AltShiftB = Alt | Shift | B,
    AltShiftC = Alt | Shift | C,
    AltShiftD = Alt | Shift | D,
    AltShiftE = Alt | Shift | E,
    AltShiftF = Alt | Shift | F,
    AltShiftG = Alt | Shift | G,
    AltShiftH = Alt | Shift | H,
    AltShiftI = Alt | Shift | I,
    AltShiftJ = Alt | Shift | J,
    AltShiftK = Alt | Shift | K,
    AltShiftL = Alt | Shift | L,
    AltShiftM = Alt | Shift | M,
    AltShiftN = Alt | Shift | N,
    AltShiftO = Alt | Shift | O,
    AltShiftP = Alt | Shift | P,
    AltShiftQ = Alt | Shift | Q,
    AltShiftR = Alt | Shift | R,
    AltShiftS = Alt | Shift | S,
    AltShiftT = Alt | Shift | T,
    AltShiftU = Alt | Shift | U,
    AltShiftV = Alt | Shift | V,
    AltShiftW = Alt | Shift | W,
    AltShiftX = Alt | Shift | X,
    AltShiftY = Alt | Shift | Y,
    AltShiftZ = Alt | Shift | Z,

    AltShiftF1 = Alt | Shift | F1,
    AltShiftF2 = Alt | Shift | F2,
    AltShiftF3 = Alt | Shift | F3,
    AltShiftF4 = Alt | Shift | F4,
    AltShiftF5 = Alt | Shift | F5,
    AltShiftF6 = Alt | Shift | F6,
    AltShiftF7 = Alt | Shift | F7,
    AltShiftF8 = Alt | Shift | F8,
    AltShiftF9 = Alt | Shift | F9,
    AltShiftF10 = Alt | Shift | F10,
    AltShiftF11 = Alt | Shift | F11,
    AltShiftF12 = Alt | Shift | F12,

    AltShiftSemicolon = Alt | Shift | Semicolon,
    AltShiftEquals = Alt | Shift | Equals,
    AltShiftComma = Alt | Shift | Comma,
    AltShiftMinus = Alt | Shift | Minus,
    AltShiftPeriod = Alt | Shift | Period,
    AltShiftSlash = Alt | Shift | Slash,
    AltShiftBacktick = Alt | Shift | Backtick,
    AltShiftLeftBracket = Alt | Shift | LeftBracket,
    AltShiftBackslash = Alt | Shift | Backslash,
    AltShiftRightBracket = Alt | Shift | RightBracket,
    AltShiftApostrophe = Alt | Shift | Apostrophe,

    AltColon = AltShiftSemicolon,
    AltPlus = AltShiftEquals,
    AltLess = AltShiftComma,
    AltUnderscore = AltShiftMinus,
    AltGreater = AltShiftPeriod,
    AltQuestion = AltShiftSlash,
    AltTilde = AltShiftBacktick,
    AltLeftBrace = AltShiftLeftBracket,
    AltVerticalBar = AltShiftBackslash,
    AltRightBrace = AltShiftRightBracket,
    AltQuote = AltShiftApostrophe,

    CtrlAltBackspace = Ctrl | Alt | Backspace,
    CtrlAltTab = Ctrl | Alt | Tab,
    CtrlAltCenter = Ctrl | Alt | Center,
    CtrlAltEnter = Ctrl | Alt | Enter,
    CtrlAltEsc = Ctrl | Alt | Esc,
    CtrlAltSpace = Ctrl | Alt | Space,
    CtrlAltPgUp = Ctrl | Alt | PgUp,
    CtrlAltPgDn = Ctrl | Alt | PgDn,
    CtrlAltEnd = Ctrl | Alt | End,
    CtrlAltHome = Ctrl | Alt | Home,
    CtrlAltLeft = Ctrl | Alt | Left,
    CtrlAltUp = Ctrl | Alt | Up,
    CtrlAltRight = Ctrl | Alt | Right,
    CtrlAltDown = Ctrl | Alt | Down,
    CtrlAltInsert = Ctrl | Alt | Insert,
    CtrlAltDelete = Ctrl | Alt | Delete,

    CtrlAlt0 = Ctrl | Alt | Key0,
    CtrlAlt1 = Ctrl | Alt | Key1,
    CtrlAlt2 = Ctrl | Alt | Key2,
    CtrlAlt3 = Ctrl | Alt | Key3,
    CtrlAlt4 = Ctrl | Alt | Key4,
    CtrlAlt5 = Ctrl | Alt | Key5,
    CtrlAlt6 = Ctrl | Alt | Key6,
    CtrlAlt7 = Ctrl | Alt | Key7,
    CtrlAlt8 = Ctrl | Alt | Key8,
    CtrlAlt9 = Ctrl | Alt | Key9,

    CtrlAltA = Ctrl | Alt | A,
    CtrlAltB = Ctrl | Alt | B,
    CtrlAltC = Ctrl | Alt | C,
    CtrlAltD = Ctrl | Alt | D,
    CtrlAltE = Ctrl | Alt | E,
    CtrlAltF = Ctrl | Alt | F,
    CtrlAltG = Ctrl | Alt | G,
    CtrlAltH = Ctrl | Alt | H,
    CtrlAltI = Ctrl | Alt | I,
    CtrlAltJ = Ctrl | Alt | J,
    CtrlAltK = Ctrl | Alt | K,
    CtrlAltL = Ctrl | Alt | L,
    CtrlAltM = Ctrl | Alt | M,
    CtrlAltN = Ctrl | Alt | N,
    CtrlAltO = Ctrl | Alt | O,
    CtrlAltP = Ctrl | Alt | P,
    CtrlAltQ = Ctrl | Alt | Q,
    CtrlAltR = Ctrl | Alt | R,
    CtrlAltS = Ctrl | Alt | S,
    CtrlAltT = Ctrl | Alt | T,
    CtrlAltU = Ctrl | Alt | U,
    CtrlAltV = Ctrl | Alt | V,
    CtrlAltW = Ctrl | Alt | W,
    CtrlAltX = Ctrl | Alt | X,
    CtrlAltY = Ctrl | Alt | Y,
    CtrlAltZ = Ctrl | Alt | Z,

    CtrlAltF1 = Ctrl | Alt | F1,
    CtrlAltF2 = Ctrl | Alt | F2,
    CtrlAltF3 = Ctrl | Alt | F3,
    CtrlAltF4 = Ctrl | Alt | F4,
    CtrlAltF5 = Ctrl | Alt | F5,
    CtrlAltF6 = Ctrl | Alt | F6,
    CtrlAltF7 = Ctrl | Alt | F7,
    CtrlAltF8 = Ctrl | Alt | F8,
    CtrlAltF9 = Ctrl | Alt | F9,
    CtrlAltF10 = Ctrl | Alt | F10,
    CtrlAltF11 = Ctrl | Alt | F11,
    CtrlAltF12 = Ctrl | Alt | F12,

    CtrlAltSemicolon = Ctrl | Alt | Semicolon,
    CtrlAltEquals = Ctrl | Alt | Equals,
    CtrlAltComma = Ctrl | Alt | Comma,
    CtrlAltMinus = Ctrl | Alt | Minus,
    CtrlAltPeriod = Ctrl | Alt | Period,
    CtrlAltSlash = Ctrl | Alt | Slash,
    CtrlAltBacktick = Ctrl | Alt | Backtick,
    CtrlAltLeftBracket = Ctrl | Alt | LeftBracket,
    CtrlAltBackslash = Ctrl | Alt | Backslash,
    CtrlAltRightBracket = Ctrl | Alt | RightBracket,
    CtrlAltApostrophe = Ctrl | Alt | Apostrophe,

    CtrlAltShiftBackspace = Ctrl | Alt | Shift | Backspace,
    CtrlAltShiftTab = Ctrl | Alt | Shift | Tab,
    CtrlAltShiftCenter = Ctrl | Alt | Shift | Center,
    CtrlAltShiftEnter = Ctrl | Alt | Shift | Enter,
    CtrlAltShiftEsc = Ctrl | Alt | Shift | Esc,
    CtrlAltShiftSpace = Ctrl | Alt | Shift | Space,
    CtrlAltShiftPgUp = Ctrl | Alt | Shift | PgUp,
    CtrlAltShiftPgDn = Ctrl | Alt | Shift | PgDn,
    CtrlAltShiftEnd = Ctrl | Alt | Shift | End,
    CtrlAltShiftHome = Ctrl | Alt | Shift | Home,
    CtrlAltShiftLeft = Ctrl | Alt | Shift | Left,
    CtrlAltShiftUp = Ctrl | Alt | Shift | Up,
    CtrlAltShiftRight = Ctrl | Alt | Shift | Right,
    CtrlAltShiftDown = Ctrl | Alt | Shift | Down,
    CtrlAltShiftInsert = Ctrl | Alt | Shift | Insert,
    CtrlAltShiftDelete = Ctrl | Alt | Shift | Delete,

    CtrlAltShift0 = Ctrl | Alt | Shift | Key0,
    CtrlAltShift1 = Ctrl | Alt | Shift | Key1,
    CtrlAltShift2 = Ctrl | Alt | Shift | Key2,
    CtrlAltShift3 = Ctrl | Alt | Shift | Key3,
    CtrlAltShift4 = Ctrl | Alt | Shift | Key4,
    CtrlAltShift5 = Ctrl | Alt | Shift | Key5,
    CtrlAltShift6 = Ctrl | Alt | Shift | Key6,
    CtrlAltShift7 = Ctrl | Alt | Shift | Key7,
    CtrlAltShift8 = Ctrl | Alt | Shift | Key8,
    CtrlAltShift9 = Ctrl | Alt | Shift | Key9,

    CtrlAltExclamation = CtrlAltShift1,
    CtrlAltAt = CtrlAltShift2,
    CtrlAltPound = CtrlAltShift3,
    CtrlAltDollar = CtrlAltShift4,
    CtrlAltPercent = CtrlAltShift5,
    CtrlAltCaret = CtrlAltShift6,
    CtrlAltAmpersand = CtrlAltShift7,
    CtrlAltAsterisk = CtrlAltShift8,
    CtrlAltLeftParen = CtrlAltShift9,
    CtrlAltRightParen = CtrlAltShift0,

    CtrlAltShiftA = Ctrl | Alt | Shift | A,
    CtrlAltShiftB = Ctrl | Alt | Shift | B,
    CtrlAltShiftC = Ctrl | Alt | Shift | C,
    CtrlAltShiftD = Ctrl | Alt | Shift | D,
    CtrlAltShiftE = Ctrl | Alt | Shift | E,
    CtrlAltShiftF = Ctrl | Alt | Shift | F,
    CtrlAltShiftG = Ctrl | Alt | Shift | G,
    CtrlAltShiftH = Ctrl | Alt | Shift | H,
    CtrlAltShiftI = Ctrl | Alt | Shift | I,
    CtrlAltShiftJ = Ctrl | Alt | Shift | J,
    CtrlAltShiftK = Ctrl | Alt | Shift | K,
    CtrlAltShiftL = Ctrl | Alt | Shift | L,
    CtrlAltShiftM = Ctrl | Alt | Shift | M,
    CtrlAltShiftN = Ctrl | Alt | Shift | N,
    CtrlAltShiftO = Ctrl | Alt | Shift | O,
    CtrlAltShiftP = Ctrl | Alt | Shift | P,
    CtrlAltShiftQ = Ctrl | Alt | Shift | Q,
    CtrlAltShiftR = Ctrl | Alt | Shift | R,
    CtrlAltShiftS = Ctrl | Alt | Shift | S,
    CtrlAltShiftT = Ctrl | Alt | Shift | T,
    CtrlAltShiftU = Ctrl | Alt | Shift | U,
    CtrlAltShiftV = Ctrl | Alt | Shift | V,
    CtrlAltShiftW = Ctrl | Alt | Shift | W,
    CtrlAltShiftX = Ctrl | Alt | Shift | X,
    CtrlAltShiftY = Ctrl | Alt | Shift | Y,
    CtrlAltShiftZ = Ctrl | Alt | Shift | Z,

    CtrlAltShiftF1 = Ctrl | Alt | Shift | F1,
    CtrlAltShiftF2 = Ctrl | Alt | Shift | F2,
    CtrlAltShiftF3 = Ctrl | Alt | Shift | F3,
    CtrlAltShiftF4 = Ctrl | Alt | Shift | F4,
    CtrlAltShiftF5 = Ctrl | Alt | Shift | F5,
    CtrlAltShiftF6 = Ctrl | Alt | Shift | F6,
    CtrlAltShiftF7 = Ctrl | Alt | Shift | F7,
    CtrlAltShiftF8 = Ctrl | Alt | Shift | F8,
    CtrlAltShiftF9 = Ctrl | Alt | Shift | F9,
    CtrlAltShiftF10 = Ctrl | Alt | Shift | F10,
    CtrlAltShiftF11 = Ctrl | Alt | Shift | F11,
    CtrlAltShiftF12 = Ctrl | Alt | Shift | F12,

    CtrlAltShiftSemicolon = Ctrl | Alt | Shift | Semicolon,
    CtrlAltShiftEquals = Ctrl | Alt | Shift | Equals,
    CtrlAltShiftComma = Ctrl | Alt | Shift | Comma,
    CtrlAltShiftMinus = Ctrl | Alt | Shift | Minus,
    CtrlAltShiftPeriod = Ctrl | Alt | Shift | Period,
    CtrlAltShiftSlash = Ctrl | Alt | Shift | Slash,
    CtrlAltShiftBacktick = Ctrl | Alt | Shift | Backtick,
    CtrlAltShiftLeftBracket = Ctrl | Alt | Shift | LeftBracket,
    CtrlAltShiftBackslash = Ctrl | Alt | Shift | Backslash,
    CtrlAltShiftRightBracket = Ctrl | Alt | Shift | RightBracket,
    CtrlAltShiftApostrophe = Ctrl | Alt | Shift | Apostrophe,

    CtrlAltColon = CtrlAltShiftSemicolon,
    CtrlAltPlus = CtrlAltShiftEquals,
    CtrlAltLess = CtrlAltShiftComma,
    CtrlAltUnderscore = CtrlAltShiftMinus,
    CtrlAltGreater = CtrlAltShiftPeriod,
    CtrlAltQuestion = CtrlAltShiftSlash,
    CtrlAltTilde = CtrlAltShiftBacktick,
    CtrlAltLeftBrace = CtrlAltShiftLeftBracket,
    CtrlAltVerticalBar = CtrlAltShiftBackslash,
    CtrlAltRightBrace = CtrlAltShiftRightBracket,
    CtrlAltQuote = CtrlAltShiftApostrophe,
};

bool keyToChar(Key key, char* pChar = nullptr);

#endif
