#include "Moves.hpp"
#include "Game.hpp"
#include "Transform.hpp"
#include <queue>
#include <algorithm>

using std::vector;
using std::queue;

MoveFinder::MoveFinder()
{
}

void MoveFinder::findMoves(const Game& game, vector<UnitState>* pMoves)
{
    const Command commands[] = {
        Command::SW,
        Command::SE,
        Command::W,
        Command::E,
        Command::CW,
        Command::CCW
    };

    uint32_t width = game.m_grid.getWidth();
    uint32_t height = game.m_grid.getHeight();
    uint32_t rots = game.m_unitInfo[game.m_curUnitIndex].m_rots;
    const vector<int32_t>& xMins =
        game.m_unitInfo[game.m_curUnitIndex].m_xMins;
    const vector<int32_t>& yMins =
        game.m_unitInfo[game.m_curUnitIndex].m_yMins;

    m_units.init(width, height, rots, xMins, yMins);
    m_moves.init(width, height, rots, xMins, yMins);

    pMoves->clear();

    typedef std::pair<UnitState, UnitRec> Entry;
    queue<Entry> unitQueue;
    {
        Entry entry;
        entry.first = game.m_curUnitState;
        entry.second.m_prevState = game.m_curUnitState;
        entry.second.m_command = Command::Invalid;
        unitQueue.push(std::move(entry));
    }

    while (!unitQueue.empty())
    {
        Entry curEntry(std::move(unitQueue.front()));
        auto& curState = curEntry.first;
        auto& curRec = curEntry.second;
        unitQueue.pop();
        if (m_units(curState).m_prevState.m_rot != (uint32_t)-1)
        {
            continue;
        }
        m_units(curState) = curRec;
        for (uint32_t iCommand = 0; iCommand < 6; iCommand++)
        {
            Command command = commands[iCommand];
            UnitState newState =
                game.transformUnit(command, game.m_curUnitIndex, curState);
            if (game.isValid(game.m_curUnitIndex, newState))
            {
                if (m_units(newState).m_prevState.m_rot == (uint32_t)-1)
                {
                    Entry entry;
                    entry.first = newState;
                    entry.second.m_prevState = curState;
                    entry.second.m_command = command;
                    unitQueue.push(std::move(entry));
                }
            }
            else if (m_moves(curState).m_prevState.m_rot == (uint32_t)-1)
            {
                UnitRec rec;
                rec.m_prevState = curState;
                rec.m_command = command;
                m_moves(curState) = rec;
                pMoves->push_back(curState);
            }
        }
    }
}

void MoveFinder::getCommands(UnitState move, vector<Command>* pCommands) const
{
    vector<Command>& commands = *pCommands;
    commands.clear();

    const UnitRec* pRec = &m_moves(move);
    while (pRec->m_command != Command::Invalid)
    {
        commands.push_back(pRec->m_command);
        pRec = &m_units(pRec->m_prevState);
    }

    std::reverse(commands.begin(), commands.end());
}
