// -*- mode:c++; c-style:k&r; c-basic-offset:4; indent-tabs-mode: nil; -*-
// vi:set et cin sw=4 cino=>se0n0f0{0}0^0\:0=sl1g0hspst0+sc3C0/0(0u0U0w0m0:

#include <iostream>
#include <cassert>
#include "PhraseSolverDP.hpp"

PhraseSolverDP::PhraseSolverDP() : m_width(0), m_height(0), m_power_words(), m_frames() { }

PhraseSolverDP::~PhraseSolverDP() { }

void PhraseSolverDP::processFrame(const PhraseInterface &pi) {
    m_width = pi.getWidth(); // assumed to be the same for all frames
    m_height = pi.getHeight(); // assumed to be the same for all frames
    auto spawn0 = pi.getStartState();
    Pos spawn = std::make_tuple(spawn0.m_pos.m_x,spawn0.m_pos.m_y,spawn0.m_rot);
    std::vector<const Pos> finals;
    for (auto &f0 : pi.getEndStates()) {
        finals.push_back(std::make_tuple(f0.m_pos.m_x,f0.m_pos.m_y,f0.m_rot));
    }
    std::cerr << "---------FRAME----------" << std::endl;
    std::cerr << "w " << pi.getWidth() << " h " << pi.getHeight() << std::endl;
    std::cerr << "spawn x " << pos_x(spawn) << " y " << pos_y(spawn) << " r " << pos_r(spawn) << std::endl;
    std::cerr << "rots " << pi.getRots() << ": ";
    for (auto r = 0; r < pi.getRots(); ++r) {
        std::cerr << r << " <" << pi.getXMin(r) << "," << pi.getYMin(r) << ">";
    }
    std::cerr << std::endl;
    std::vector<const Pos> invalids;
    for (int y = 0; y < pi.getHeight(); ++y) {
        for (int x = 0; x < pi.getWidth(); ++x) {
            for (int r = 0; r < 6; ++r) {
                UnitState us(x,y,r);
                if (!pi.isValid(us)) {
                    invalids.push_back(std::make_tuple(x,y,r));
                }
            }
        }
    }
    std::vector<int> x_mins;
    std::vector<int> y_mins;
    for (auto r = 0; r < pi.getRots(); ++r) {
        x_mins.push_back(pi.getXMin(r));
        y_mins.push_back(pi.getYMin(r));
    }
    int r_lim = pi.getRots();
    Frame f = { spawn, finals, invalids, r_lim, x_mins, y_mins };
    m_frames.push_back(f);
    for (auto w : pi.getPhrases()) {
        m_power_words.insert(w);
    }
}

void PhraseSolverDP::getChars(std::vector<char> *pChars) {
    pChars->clear();
    std::vector<std::tuple<const std::string,int> > power_words;
    for (auto w : m_power_words) {
        power_words.push_back(std::make_tuple(w, w.length() * 2));
    }
    Vocabulary vocab(power_words);
    assert(m_frames.size() > 0);
    Encoder enc(m_width, m_height, vocab, m_frames);
    auto solution = enc.run();
    for (auto c : std::get<1>(solution)) pChars->push_back(c);
}

