#ifndef STATETABLE_HPP
#define STATETABLE_HPP

#include "Common.hpp"
#include "Grid.hpp"
#include "UnitState.hpp"

template<typename T>
class StateTable
{
public:
    StateTable() : m_data(), m_xMins(), m_yMins() {}

    StateTable(StateTable&& other) :
        m_data(std::move(other.m_data)),
        m_xMins(std::move(other.m_xMins)),
        m_yMins(std::move(other.m_yMins))
    {
        other.m_data.clear();
        other.m_xMins.clear();
        other.m_yMins.clear();
    }

    ~StateTable() {}

    StateTable& operator=(StateTable&& other)
    {
        if (this != &other)
        {
            m_data = std::move(other.m_data);
            m_xMins = std::move(other.m_xMins);
            m_yMins = std::move(other.m_yMins);
            other.m_data.clear();
            other.m_xMins.clear();
            other.m_yMins.clear();
        }
        return *this;
    }

    void init(uint32_t width, uint32_t height, uint32_t rots,
              const std::vector<int32_t>& xMins,
              const std::vector<int32_t>& yMins)
    {
        m_data.clear();
        m_data.resize(rots);
        for (uint32_t iRot = 0; iRot < rots; iRot++)
        {
            m_data[iRot].resize(width + 2, height);
        }
        m_xMins = xMins;
        m_yMins = yMins;
    }

    const T& operator()(int32_t x, int32_t y, uint32_t rot) const
    {
#if 0
        if (rot >= m_data.size())
        {
            fprintf(stderr, "rot out of range\n");
            exit(1);
        }
        if (x < m_xMins[rot] || x >= m_xMins[rot] + (int32_t)m_data[rot].getWidth() + 2)
        {
            fprintf(stderr, "x out of range\n");
            exit(1);
        }
        if (y < m_yMins[rot] || y >= m_yMins[rot] + (int32_t)m_data[rot].getHeight())
        {
            fprintf(stderr, "y out of range\n");
            exit(1);
        }
#endif
        return m_data[rot](x - m_xMins[rot], y - m_yMins[rot]);
    }
    T& operator()(int32_t x, int32_t y, uint32_t rot)
    {
#if 0
        if (rot >= m_data.size())
        {
            fprintf(stderr, "rot out of range\n");
            exit(1);
        }
        if (x < m_xMins[rot] || x >= m_xMins[rot] + (int32_t)m_data[rot].getWidth() + 2)
        {
            fprintf(stderr, "x out of range\n");
            exit(1);
        }
        if (y < m_yMins[rot] || y >= m_yMins[rot] + (int32_t)m_data[rot].getHeight())
        {
            fprintf(stderr, "y out of range\n");
            exit(1);
        }
#endif
        return m_data[rot](x - m_xMins[rot], y - m_yMins[rot]);
    }

    const T& operator()(UnitState unitState) const
    {
        return operator()(unitState.m_pos.m_x,
                          unitState.m_pos.m_y,
                          unitState.m_rot);
    }
    T& operator()(UnitState unitState)
    {
        return operator()(unitState.m_pos.m_x,
                          unitState.m_pos.m_y,
                          unitState.m_rot);
    }

    void swap(StateTable& other)
    {
        using std::swap;
        swap(m_data, other.m_data);
        swap(m_xMins, other.m_xMins);
        swap(m_yMins, other.m_yMins);
    }

public:
    std::vector<Grid<T>> m_data;
    std::vector<int32_t> m_xMins;
    std::vector<int32_t> m_yMins;

private:
    StateTable(const StateTable& other) = delete;
    StateTable& operator=(const StateTable& other) = delete;
};

template<class T>
void swap(StateTable<T>& x, StateTable<T>& y)
{
    x.swap(y);
}

#endif
