#include "Utils.hpp"

#if PLATFORM_WINDOWS
#include <windows.h>
#else
#include <chrono>
#endif

using std::string;

void sleepS(uint32_t timeS)
{
    sleepMS(timeS * 1000);
}

void sleepMS(uint32_t timeMS)
{
#if PLATFORM_WINDOWS
    Sleep(timeMS);
#else
    struct timespec req;
    memset(&req, 0, sizeof(req));
    req.tv_sec = (time_t)(timeMS / 1000);
    req.tv_nsec = (long)((timeMS % 1000) * 1000000);
    nanosleep(&req, nullptr);
#endif
}

uint64_t getTicksS()
{
#if PLATFORM_WINDOWS
    return (uint64_t)GetTickCount64() / UINT64_C(1000);
#else
    return (std::chrono::duration_cast<std::chrono::seconds>
            (std::chrono::system_clock::now().time_since_epoch()).count());
#endif
}

uint64_t getTicksMS()
{
#if PLATFORM_WINDOWS
    return (uint64_t)GetTickCount64();
#else
    return (std::chrono::duration_cast<std::chrono::milliseconds>
            (std::chrono::system_clock::now().time_since_epoch()).count());
#endif
}

string strprintf(const char* format, ...)
{
    va_list ap;
    va_start(ap, format);
    string str = vstrprintf(format, ap);
    va_end(ap);
    return str;
}

string vstrprintf(const char* format, va_list ap)
{
    va_list ap2;
    va_copy(ap2, ap);
    size_t size = vsnprintf(nullptr, 0, format, ap2);
    va_end(ap2);
    std::unique_ptr<char[]> buf(new char[size+1]);
    vsnprintf(buf.get(), size+1, format, ap);
    return buf.get();
}

int strcmpNoCase(const char* s1, const char* s2)
{
    while (true)
    {
        char ch1 = *s1;
        char ch2 = *s2;
        if (ch1 == '\0')
        {
            if (ch2 == '\0')
                return 0;
            else
                return -1;
        }
        if (ch2 == '\0')
            return 1;
        if (ch1 >= 'a' && ch1 <= 'z')
            ch1 = 'A' + (ch1 - 'a');
        if (ch2 >= 'a' && ch2 <= 'z')
            ch2 = 'A' + (ch2 - 'a');
        if (ch1 != ch2)
        {
            if (ch1 < ch2)
                return -1;
            else
                return 1;
        }
        s1++;
        s2++;
    }
}
