// -*- mode:c++; c-style:k&r; c-basic-offset:4; indent-tabs-mode: nil; -*-
// vi:set et cin sw=4 cino=>se0n0f0{0}0^0\:0=sl1g0hspst0+sc3C0/0(0u0U0w0m0:

#include <cassert>
#include <iostream>
#include "Vocabulary.hpp"

Vocabulary::Vocabulary() : m_words(), m_texts(), m_decode_tbl(128, -1) {
    auto dbg = false;
    // NB: deltas are for forward directions
    m_primitives.push_back(Word(std::make_tuple(-1,0,0), 0)); // W
    m_primitives.push_back(Word(std::make_tuple(1,0,0), 0)); // E
    m_primitives.push_back(Word(std::make_tuple(-1,1,0), 0)); // SW
    m_primitives.push_back(Word(std::make_tuple(0,1,0), 0)); // SE
    m_primitives.push_back(Word(std::make_tuple(0,0,1), 0)); // CW
    m_primitives.push_back(Word(std::make_tuple(0,0,-1), 0)); // CCW

    // NB: deltas are for reversed directions
    m_words.push_back(Word(std::make_tuple(1,0,0), 0)); m_texts.push_back(dbg ? "W " : "p"); // W
    m_words.push_back(Word(std::make_tuple(-1,0,0), 0)); m_texts.push_back(dbg ? "E " : "b"); // E
    m_words.push_back(Word(std::make_tuple(1,-1,0), 0)); m_texts.push_back(dbg ? "SW " : "a"); // SW
    m_words.push_back(Word(std::make_tuple(0,-1,0), 0)); m_texts.push_back(dbg ? "SE " : "l"); // SE
    m_words.push_back(Word(std::make_tuple(0,0,-1), 0)); m_texts.push_back(dbg ? "CW " : "d"); // CW
    m_words.push_back(Word(std::make_tuple(0,0,1), 0)); m_texts.push_back(dbg ? "CCW " : "k"); // CCW

    for (auto c : "p'!.03") m_decode_tbl[c] = 0; // W
    for (auto c : "P") m_decode_tbl[c] = 0; // W
    for (auto c : "bcefy2") m_decode_tbl[c] = 1; // E
    for (auto c : "BCEFY") m_decode_tbl[c] = 1; // E
    for (auto c : "aghij4") m_decode_tbl[c] = 2; // SW
    for (auto c : "AGHIJ") m_decode_tbl[c] = 2; // SW
    for (auto c : "lmno 5") m_decode_tbl[c] = 3; // SE
    for (auto c : "LMNO") m_decode_tbl[c] = 3; // SE
    for (auto c : "dqrvz1") m_decode_tbl[c] = 4; // CW
    for (auto c : "DQRVZ") m_decode_tbl[c] = 4; // CW
    for (auto c : "kstuwx") m_decode_tbl[c] = 5; // CCW
    for (auto c : "KSTUWX") m_decode_tbl[c] = 5; // CCW
}

Vocabulary::Vocabulary(const std::vector<std::tuple<const std::string,int> > &power_words) : Vocabulary() {
    for (auto pw : power_words) {
        auto &s = std::get<0>(pw);
        assert(s.size() > 0);
        assert((s.size() > 1) || (m_decode_tbl[s[0]] < 0)); // don't overwrite primitives
        std::vector<const Delta> ds;
        for (auto c = s.crbegin(); c != s.crend(); ++c) {
            auto ix = m_decode_tbl[*c];
            assert(ix >= 0);
            auto &w = m_words[ix];
            for (auto &d : w.rpath) { ds.push_back(d); }
        }
        m_words.push_back(Word(ds, std::get<1>(pw)));
        m_texts.push_back(s);
    }
}

